#!/usr/bin/python
import datetime, time, logging, minimalmodbus, requests
from requests.exceptions import Timeout

vpsTimeout = (0.5, 0.5)

wn = minimalmodbus.Instrument('/dev/ttyUSB0', 1)
wn.serial.baudrate = 9600

logging.basicConfig()

def read_meter():
    try:
        LEG1 = int(wn.read_float(1011)) # Watts
        LEG2 = int(wn.read_float(1013))
        GENW = int(wn.read_float(1015)) # PV - W
        CONS = int(wn.read_float(1009)) # Consuming now - W
        IMPA = round(wn.read_float(1107), 3) # Import - kWh
        IMPB = round(wn.read_float(1109), 3)
        PVWH = round(wn.read_float(1111), 3) # PV - kWh
        NETA = round(wn.read_float(1101), 3)
        NETB = round(wn.read_float(1103), 3)
        EXPA = round(wn.read_float(1117), 3) # Export - kWh
        EXPB = round(wn.read_float(1119), 3)
        TECO = round(wn.read_float(1001), 1) # Total Energy Consumed
        L1PF = round(wn.read_float(1141), 4)
        L2PF = round(wn.read_float(1143), 4)
        PVPF = round(wn.read_float(1145), 4)
        VOLT = round(wn.read_float(1025), 3)

        TOTL = CONS - GENW
        IMPT = IMPA + IMPB
        EXPT = EXPA + EXPB
        NETT = TECO - PVWH

        IMPW = 0
        if TOTL > 0:
            IMPW = TOTL

        DRCT = CONS - IMPW
        EXPW = GENW - DRCT

        meas = {'LEG1':LEG1,
            'LEG2':LEG2,
            'GENW':GENW,
            'IMPA':IMPA,
            'IMPB':IMPB,
            'EXPA':EXPA,
            'EXPB':EXPB,
            'PVWH':PVWH,
            'CONS':CONS,
            'TOTL':TOTL,
            'TECO':TECO,
            'IMPT':IMPT,
            'EXPT':EXPT,
            'NETT':NETT,
            'VOLT':VOLT,
            'L1PF':L1PF,
            'L2PF':L2PF,
            'DRCT':DRCT,
            'IMPW':IMPW,
            'EXPW':EXPW,
            'PVPF':PVPF
        }

        tmpData = []
        for key, val in meas.items():
            tmpData.append('{} value={} {}'.format(key, val, int(time.time())))
            Data = '\n'.join(tmpData)

        requests.post('http://140.174.6.17:8086/write?db=RPP&precision=s', data=Data, timeout=vpsTimeout)
    except Timeout:
        pass

while True:
    try:
        read_meter()
        time.sleep(5.0 - (time.time() % 5.0))
    except (KeyboardInterrupt, SystemExit):
        raise

